LABELS = {
    "storage.googleapis.com": "Google API Storage",
    "v.vrv.co": "VRV",
    "pl.crunchyroll.com": "Crunchyroll",
}

SOURCE_REPOSITORY = "justfoolingaround", "animdl"
VERSION_FILE_PATH = "master", "animdl/core/__version__.py"
